local hcTk = HCToolkit

function hcTk.MaidenEncounterStart()
    for k, v in pairs(hcTk.Group) do
        hcTk.Group[k]["Assigned"] = false
        hcTk.Group[k]["FelDebuff"] = false
        hcTk.Group[k]["LightDebuff"] = false
    end
    hcTk.Encounter["FelFront"] = {}
    hcTk.Encounter["FelBack"] = {}
    hcTk.Encounter["LightFront"] = {}
    hcTk.Encounter["LightBack"] = {}
end

function hcTk.MaidenEncounterEnd()
    for k, v in pairs(hcTk.Group) do
        hcTk.Group[k]["Assigned"] = nil
        hcTk.Group[k]["FelDebuff"] = nil
        hcTk.Group[k]["LightDebuff"] = nil
    end
    hcTk.Encounter["FelFront"] = nil
    hcTk.Encounter["FelBack"] = nil
    hcTk.Encounter["LightFront"] = nil
    hcTk.Encounter["LightBack"] = nil
end

function hcTk.MaidenCombatLog(...)
    local msg = select(2, ...)
    if (hcTk.MaidenCombatLogFns[msg] ~= nil) then
        hcTk.MaidenCombatLogFns[msg](...)
    end
end

function hcTk.MaidenSpellAuraApplied(...)
    local destGUID, _, _, _, spellId = select(8, ...)
    if (hcTk.Group[destGUID] ~= nil) then
        if (spellId == 235240) then
            hcTk.Group[destGUID]["FelDebuff"] = true
            hcTk.Group[destGUID]["LightDebuff"] = false
        elseif (spellId == 235213) then
            hcTk.Group[destGUID]["FelDebuff"] = false
            hcTk.Group[destGUID]["LightDebuff"] = true
        end
    end
end

function hcTk.MaidenSpellCastSuccess(...)
    local spellId = select(12, ...)
    if (spellId == 235271) then -- Infusion SpellCastSuccess
        hcTk.MaidenWipeSoaks()
        C_Timer.After(2, hcTk.MaidenAssignSoaks)
    end
end

function hcTk.MaidenUnitDied(...)
    local destGUID = select(8, ...)
    if (hcTk.Group[destGUID] ~= nil) then
        hcTk.Group[destGUID]["Assigned"] = false
        hcTk.Group[destGUID]["FelDebuff"] = false
        hcTk.Group[destGUID]["LightDebuff"] = false
        -- hcTk.MaidenAssignSoaks()
    end
end

function hcTk.MaidenAssignSoaks()
    local e = hcTk.Encounter
    local felColor = "FF00FF00"
    local lightColor = "FFFFDF00"
    if (e["FelFront"] == nil or e["LightFront"] == nil or e["FelBack"] == nil or e["LightBack"] == nil) then return false end
    hcTk.MaidenWipeSoaks()
    hcTk.MaidenAssignSoakSection(e["FelFront"], {[1] = "Tank", [2] = "MeleeDPS", [3] = "RangedDPS", [4] = "Healer"}, "FelDebuff")
    hcTk.MaidenAssignSoakSection(e["LightFront"], {[1] = "Tank", [2] = "MeleeDPS", [3] = "RangedDPS", [4] = "Healer"}, "LightDebuff")
    hcTk.MaidenAssignSoakSection(e["FelBack"], {[1] = "Healer", [2] = "RangedDPS", [3] = "MeleeDPS", [4] = "Tank"}, "FelDebuff")
    hcTk.MaidenAssignSoakSection(e["LightBack"], {[1] = "Healer", [2] = "RangedDPS", [3] = "MeleeDPS", [4] = "Tank"}, "LightDebuff")
    hcTk.MaidenPrintAssignments({[1] = e["FelFront"], [2] = e["FelBack"], [3] = e["LightFront"], [4] = e["LightBack"]}, {[1] = "Fel Front", [2] = "Fel Back", [3] = "Light Front", [4] = "Light Back"}, {[1] = "Outer", [2] = "Middle", [3] = "Inner"}, {[1] = felColor, [2] = felColor, [3] = lightColor, [4] = lightColor})
end

function hcTk.MaidenAssignSoakSection(section, roles, qualifier)
    for _, r in ipairs(roles) do
        if (hcTk.MaidenAssignSoakSectionByRole(section, r, qualifier)) then return true end
    end
end

function hcTk.MaidenAssignSoakSectionByRole(section, r, qualifier)
    for k, guid in ipairs(hcTk.GroupKeys) do
        if (k > 20) then return false end -- Add Support For Heroic & Mythic
        if (hcTk.Group[guid]["role"] == r and not hcTk.Group[guid]["Assigned"] and hcTk.Group[guid][qualifier]) then
            section[#section + 1] = guid
            hcTk.Group[guid]["Assigned"] = true
            if #section >= 3 then return true end
        end
    end
    return false
end

function hcTk.MaidenPrintAssignments(sections, sectionNames, positions, colors)
    local str = "Maiden Intermission Assignments (Outer | Middle | Inner)\n"
    for sKey, s in ipairs(sections) do
        str = str .. string.format("|c%s%s:|r ", colors[sKey], sectionNames[sKey]) .. ""
        for k, guid in ipairs(s) do
            str = str .. hcTk.UnitPlayerString(hcTk.Group[guid]) .. " | "
            if (guid == hcTk.PlayerGUID) then
                hcTk.MaidenPrintPersonalAssignment(colors[sKey], positions[k], sectionNames[sKey])
            end
        end
        --str = string.format("%s\n", str:sub(1, -4))
        str = str .. "\n"
    end
    -- Add Option & Check
    hcTk.FrameUpdateText(HCToolkitGroupFrame, str)
end

function hcTk.MaidenPrintPersonalAssignment(color, position, section)
    -- Add Option & Check
    hcTk.FrameUpdateText(HCToolkitPersonalFrame, string.format("|c%s%s - %s|r", color, position, section))
end

function hcTk.MaidenWipeSoaks()
    for k, v in pairs(hcTk.Group) do
        hcTk.Group[k]["Assigned"] = false
    end
    hcTk.Encounter["FelFront"] = {}
    hcTk.Encounter["FelBack"] = {}
    hcTk.Encounter["LightFront"] = {}
    hcTk.Encounter["LightBack"] = {}
end

hcTk.MaidenCombatLogFns = {
    ["SPELL_AURA_APPLIED"] = hcTk.MaidenSpellAuraApplied,
    ["SPELL_CAST_SUCCESS"] = hcTk.MaidenSpellCastSuccess,
    ["UNIT_DIED"] = hcTk.MaidenUnitDied
}